package org.apache.maven.linkcheck.validation;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Maven" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Maven", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * ====================================================================
 */
 
import java.io.File;

/**
 * A link validator solely for files on the local filesystem.
 * 
 * @author <a href="mailto:bwalding@apache.org">Ben Walding</a>
 * @version $Id: FileLinkValidator.java,v 1.2 2003/02/03 14:13:54 bwalding Exp $
 */
public class FileLinkValidator implements LinkValidator {
    private final static LinkValidationResult LVR_INVALID =
        new LinkValidationResult(LinkValidationResult.INVALID, false);
        
    private final static LinkValidationResult LVR_VALID =
        new LinkValidationResult(LinkValidationResult.VALID, false);

    /**
     * @see org.apache.maven.linkcheck.LinkValidator#validateLink(org.apache.maven.linkcheck.LinkValidationItem)
     */
    public LinkValidationResult validateLink(LinkValidationItem lvi) {
        File f = getFile(lvi);

        if (f.exists())
            return LVR_VALID;
        else
            return LVR_INVALID;
    }

    /**
     * 
     * @param lvi
     * @return File
     */
    protected File getFile(LinkValidationItem lvi) {
        String link = lvi.getLink();
        if (link.indexOf('#') != -1) {
            link = link.substring(0, link.indexOf('#'));

            //If the link was just #fred or similar, then the file is the file it came from
            //XXX: Theoretically we could even validate the anchor tag?
            if (link.trim().length() == 0)
                return lvi.getSource();
        }

        File f = new File(lvi.getSource().getParentFile(), link);
        return f;
    }

    /**
     * @see org.apache.maven.linkcheck.LinkValidator#getResourceKey(org.apache.maven.linkcheck.LinkValidationItem)
     */
    public Object getResourceKey(LinkValidationItem lvi) {
        String link = lvi.getLink();

        if (link.indexOf(':') != -1)
            return null;

        return getFile(lvi).getAbsolutePath();
    }

}
